# EdwardsActiveDigitalController by JB@KIT 09/2014
# USB interface for Edwards Controller
# modified for using with Raspberry Pi 03/2015
# pressure readout device @ UFO: G1, G2
# model: D39591000 (enhanced)

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import numpy as np
import time,sys
import atexit
import serial

class Edwards_p_gauge(object):
        
    def __init__(self, device='/dev/ttyUSB1'):
    
        #Instrument.__init__(self, name, tags=['physical'])
        '''
        self.add_parameter('condenser_pressure',
            type=types.FloatType,
            flags=Instrument.FLAG_GET, units='mbar')
        self.add_parameter('still_pressure',
            type=types.FloatType,
            flags=Instrument.FLAG_GET, units='mbar')
        '''
        # open serial port, 9600, 8,N,1, timeout 1s
        #device = address   #"COM4"
        baudrate = 9600
        timeout = 1
        # Port B on the USB_to_serial converter
        #device = "/dev/ttyUSB1"
        self.ser = self._std_open(device,baudrate,timeout)
        atexit.register(self.ser.close)
        
    def serial_close(self):
        self.ser.close()
        
    def _std_open(self,device,baudrate,timeout):
        return serial.Serial(device, baudrate, timeout=timeout)
                
    def remote_cmd(self,cmd):
        cmd+='\r\n'
        time.sleep(0.3)
        self.ser.write(cmd)
        
        time.sleep(0.5)   #wait 0.5s
        #value = self.ser.readline().strip("\x06")
        rem_char = self.ser.inWaiting()
        
        value = self.ser.read(rem_char) # .strip("\x06")
        #print "##"+value.strip()+"###"
        try:
            return float(value.strip())
        except Exception as m:
            print m, value.strip()
            return 0

    #read out status
    def get_condenser_pressure(self):
        try:
            return self.remote_cmd("?GA1")
        except Exception as detail:
            print "Error: ",detail
            return 0
            
    def get_still_pressure(self):
        try:
            return self.remote_cmd("?GA2")
        except Exception as detail:
            print "Error: ",detail
            return 0
    
    def get_all(self):
        return self.get_condenser_pressure(), self.get_still_pressure()

if __name__ == "__main__":   #if executed as main (and not imported)
    time.sleep(0.1) 
    rd = Edwards_p_gauge()
    print str(rd.get_all())
 


# Bluefors_GHS.py
# S1@KIT 10/2020
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from qkit.core.instrument_base import Instrument
import socket
import logging
import time

class Bluefors_GHS(Instrument):
    """
    This is the python driver to read valve status and pressures from the
    Bluefors control PC

    Usage:
    Initialise with
    <name> = instruments.create('<name>', host='<IP address>', port=<port>)
    """
    
    def __init__(self, name, host, port):
        '''
        Initializes

        Input:
            name (string)    : name of the instrument
            address (string) : IP address
            port (int)       : port
        '''
        
        logging.info(__name__ + ' : Initializing instrument')
        Instrument.__init__(self, name, tags=['physical'])
        
        self._host = host
        self._port = port
        self._reconnect()
        self._wait_time = .05
        
        # Implement parameters
        self.add_parameter('pressure', type=float,flags=Instrument.FLAG_GET, channels=(1, 6), channel_prefix='p%d_',units='mbar')
        self.add_parameter('flowmeter', type=float,flags=Instrument.FLAG_GET,units='mmol/s')
        self.add_parameter('state', type=bool, flags=Instrument.FLAG_GET, channel_prefix="%s_",
                           channels=["v%i"%i for i in range(1,22)]+["compressor","ext","hs-mc","hs-still","pulsetube","scroll1","scroll2","turbo1"])
        
        
        self.get_all()
        
    def get_all(self):
        pass
    ###
    # Communication with device
    ###
    
    def do_get_pressure(self,channel):
        r = self._ask("mgstatus %i"%channel)
        return float(r.split(":")[1].strip())
    
    def do_get_flowmeter(self):
        r = self._ask("fmstatus")
        return float(r.split(":")[1].strip())
    
    def do_get_state(self,channel):
            r = self._ask("status %s" % channel)
            return bool(int(r.split("=")[1].strip()))

    def _ask(self,command):
        #print(command)
        if type(command) is not bytes:
            command = bytes(str(command), "utf-8")
        command += b"\r\n"
        try:
            self._soc.send(command)
            time.sleep(self._wait_time)
            data = self._soc.recv(1024)
            return data.decode().strip()
        except ConnectionAbortedError:
            self._reconnect()
            return self._ask(command)
            
    
    def _reconnect(self):
        logging.info(__name__+": Bluefors_GHS: reconnecting to control PC.")
        for i in range(5):
            try:
                if i>1:
                    print(__name__+": waiting 30 seconds before next reconnect attempt.")
                    time.sleep(30)
                self._soc = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                self._soc.connect((self._host, self._port))
                self._soc.settimeout(5)
                self._soc.send(b"test\r\n")
                time.sleep(.5)
                self._soc.recv(1024)
                return True
            except Exception as e:
                if i>=5:
                    raise e
                else:
                    print(e)


# Anritsu_VNA.py 
# Pascal Macha <pascalmacha@googlemail.com>, 2010
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from qkit.core.instrument_base import Instrument
from qkit import visa
import types
import logging
import numpy

class Anritsu_VNA_MS4642B(Instrument):
    '''
    This is the python driver for the Anritsu MS4642B Vector Network Analyzer

    Usage:
    Initialise with
    <name> = instruments.create('<name>', address='<GPIB address>', reset=<bool>)
    
    '''

    def __init__(self, name, address, channel_index = 1):
        '''
        Initializes 

        Input:
            name (string)    : name of the instrument
            address (string) : GPIB address
        '''
        
        logging.info(__name__ + ' : Initializing instrument')
        Instrument.__init__(self, name, tags=['physical'])

        self._address = address
        self._visainstrument = visa.instrument(self._address)
        self._zerospan = False
        self._freqpoints = 0
        self._ci = channel_index 
        self._start = 0
        self._stop = 0
        self._nop = 0

        # Implement parameters
        self.add_parameter('nop', type=int,
            flags=Instrument.FLAG_GETSET,
            minval=1, maxval=25000,
            tags=['sweep'])

        self.add_parameter('bandwidth', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=1e9,
            units='Hz', tags=['sweep']) 

        self.add_parameter('averages', type=int,
            flags=Instrument.FLAG_GETSET,
            minval=1, maxval=1024, tags=['sweep'])                    

        self.add_parameter('Average', type=bool,
            flags=Instrument.FLAG_GETSET)   

        self.add_parameter('centerfreq', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=20e9,
            units='Hz', tags=['sweep'])

        self.add_parameter('cwfreq', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=20e9,
            units='Hz', tags=['sweep'])

        self.add_parameter('startfreq', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=20e9,
            units='Hz', tags=['sweep'])            

        self.add_parameter('stopfreq', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=20e9,
            units='Hz', tags=['sweep'])                        

        self.add_parameter('span', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=20e9,
            units='Hz', tags=['sweep'])        

        self.add_parameter('power', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=-30, maxval=15,
            units='dBm', tags=['sweep'])

        self.add_parameter('cw', type=bool,
            flags=Instrument.FLAG_GETSET)

        self.add_parameter('zerospan', type=bool,
            flags=Instrument.FLAG_GETSET)

        self.add_parameter('channel_index', type=int,
            flags=Instrument.FLAG_GETSET)     

        self.add_parameter('source_attenuation', type=int,
            flags=Instrument.FLAG_GETSET,channels=(1,2),
            minval=0, maxval=60,
            units='dB', tags=['sweep'])
            
        self.add_parameter('source_power_start', type=float,
            flags=Instrument.FLAG_GETSET,channels=(1,2),
            minval=-2.9e1, maxval=3e1,
            units='dBm', tags=['sweep'])
            
        self.add_parameter('source_power_stop', type=float,
            flags=Instrument.FLAG_GETSET,channels=(1,2),
            minval=-2.9e1, maxval=3e1,
            units='dBm', tags=['sweep'])
            
        self.add_parameter('calibration_state', type=bool,
            flags=Instrument.FLAG_GETSET) 
        
        self.add_parameter('sweep_type', type=str,
            flags=Instrument.FLAG_GETSET,tags=['sweep']) 
            
        self.add_parameter('power_nop', type=int,
            flags=Instrument.FLAG_GETSET,channels=(1,2),
            minval=0, maxval=60,
            tags=['sweep'])
        
        self.add_parameter('avg_type', type=str,
            flags=Instrument.FLAG_GETSET,tags=['sweep']) 
        
        self.add_parameter('edel', type=float,   #added by MW
            flags=Instrument.FLAG_GETSET,
            minval=-1, maxval=1,
            units='s', tags=['sweep'])
        
        self.add_parameter('sweeptime', type=float,   #JB
            flags=Instrument.FLAG_GET,
            minval=0, maxval=1e-3,
            units='s', tags=['sweep'])
            
        self.add_parameter('sweeptime_averages', type=float,   #JB
            flags=Instrument.FLAG_GET,
            minval=0, maxval=1e-3,
            units='s', tags=['sweep'])
            
        self.add_parameter('sweep_mode', type=str,
            flags=Instrument.FLAG_GETSET,tags=['sweep']) 
                    
        #Triggering Stuff
        self.add_parameter('trigger_source', type=str,
            flags=Instrument.FLAG_GETSET)
        
        # Implement functions
        self.add_function('get_freqpoints')
        self.add_function('get_tracedata')
        self.add_function('init')
        self.add_function('returnToLocal')
        self.add_function('set_edel_auto')
        self.add_function('pre_measurement')
        self.add_function('start_measurement')
        self.add_function('ready')
        self.add_function('post_measurement')
        self.add_function('avg_clear')
        self.add_function('avg_status')
        
        #self._oldspan = self.get_span()
        #self._oldnop = self.get_nop()
        #if self._oldspan==0.002:
        #  self.set_zerospan(True)
        
        self.get_all()
    
    def get_all(self):        
        self.get_nop()
        self.get_power()
        self.get_centerfreq()
        self.get_startfreq()
        self.get_stopfreq()
        self.get_span()
        self.get_bandwidth()
        self.get_trigger_source()
        self.get_Average()
        self.get_averages()
        self.get_freqpoints()   
        self.get_channel_index()
        self.get_cw()
        self.get_source_attenuation1()
        self.get_source_attenuation2()
        self.get_source_power_start1()
        self.get_source_power_start2()
        self.get_source_power_stop1()
        self.get_source_power_stop2()
        self.get_calibration_state()
        self.get_sweep_type()
        self.get_power_nop1()
        self.get_power_nop2()
        self.get_avg_type()
        self.get_edel()
        #self.get_zerospan()
        
    ###
    #Communication with device
    ### 
    
    def init(self):
        if self._zerospan:
          self._visainstrument.write('INIT1;*wai')
        else:
          if self.get_Average():
            for i in range(self.get_averages()):            
              self._visainstrument.write('INIT1;*wai')
          else:
              self._visainstrument.write('INIT1;*wai')

        
    def avg_clear(self):
        self._visainstrument.write(':SENS%i:AVER:CLE' %(self._ci))
    
    def returnToLocal(self):
        self._visainstrument.write('RTL')
    
    def avg_status(self):
        return int(self._visainstrument.ask(':SENS%i:AVER:SWE?' %(self._ci)))   
        
    def get_tracedata(self, format = 'AmpPha', single=False):
        '''
        Get the data of the current trace

        Input:
            format (string) : 'AmpPha': Amp in dB and Phase, 'RealImag',

        Output:
            'AmpPha':_ Amplitude and Phase
        '''
        #data = self._visainstrument.ask_for_values(':FORMAT REAL,32;*CLS;CALC1:DATA:NSW? SDAT,1;*OPC',format=1)      
        data = self._visainstrument.ask_for_values('FORM:DATA REAL; FORM:BORD SWAPPED; CALC%i:SEL:DATA:SDAT?'%(self._ci), fmt = 3)      
        data_size = numpy.size(data)
        datareal = numpy.array(data[0:data_size:2])
        dataimag = numpy.array(data[1:data_size:2])
          
        if format.upper() == 'REALIMAG':
          if self._zerospan:
            return numpy.mean(datareal), numpy.mean(dataimag)
          else:
            return datareal, dataimag
        elif format.upper() == 'AMPPHA':
          if self._zerospan:
            datareal = numpy.mean(datareal)
            dataimag = numpy.mean(dataimag)
            dataamp = numpy.sqrt(datareal*datareal+dataimag*dataimag)
            datapha = numpy.arctan(dataimag/datareal)
            return dataamp, datapha
          else:
            dataamp = numpy.sqrt(datareal*datareal+dataimag*dataimag)
            datapha = numpy.arctan2(dataimag,datareal)
            return dataamp, datapha
        else:
          raise ValueError('get_tracedata(): Format must be AmpPha or RealImag') 
      
    def get_freqpoints(self, query = False):      
      if query == True:        
        self._freqpoints = self._visainstrument.ask_for_values('FORM:DATA REAL; FORM:BORD SWAPPED; :SENS%i:FREQ:DATA?'%(self._ci), fmt = visa.double)
      
        #self._freqpoints = numpy.array(self._visainstrument.ask_for_values('SENS%i:FREQ:DATA:SDAT?'%self._ci,format=1)) / 1e9
        #self._freqpoints = numpy.array(self._visainstrument.ask_for_values(':FORMAT REAL,32;*CLS;CALC1:DATA:STIM?;*OPC',format=1)) / 1e9
      else:
        self._freqpoints = numpy.linspace(self._start,self._stop,self._nop)
      return self._freqpoints
      
    def get_powerpoints(self):      
        self._powerpoints = numpy.linspace(self.get_source_power_start1(),self.get_source_power_stop1(),self.get_power_nop1())
        return self._powerpoints


    ###
    # SET and GET functions
    ###
    
    def do_set_sweep_mode(self, mode):
        '''
        select the sweep mode from 'hold', 'cont', single'
        '''
        if mode == 'hold':
            self._visainstrument.write(':SENS:HOLD:FUNC HOLD')
        elif mode == 'cont':
            self._visainstrument.write(':SENS:HOLD:FUNC CONT')
        elif mode == 'single':
            self._visainstrument.write(':SENS:HOLD:FUNC SING')
        else:
            logging.warning('invalid mode')
            
    def do_get_sweep_mode(self):
        return self._visainstrument.ask(':SENS:HOLD:FUNC ?')
    
    def do_set_nop(self, nop):
        '''
        Set Number of Points (nop) for sweep

        Input:
            nop (int) : Number of Points

        Output:
            None
        '''
        logging.debug(__name__ + ' : setting Number of Points to %s ' % (nop))
        if self._zerospan:
          print('in zerospan mode, nop is 1')
        else:
          cw = self.get_cw()
          if(nop == 1) and (not cw):
              logging.info(__name__ + 'nop == 1 is only supported in CW mode.')
              self.set_cw(True)
          if(cw):
            self._visainstrument.write(':SENS%i:SWE:CW:POIN %i' %(self._ci,nop))
          else:
            self._visainstrument.write(':SENS%i:SWE:POIN %i' %(self._ci,nop))
          self._nop = nop
          self.get_freqpoints() #Update List of frequency points  
        
    def do_get_nop(self):
        '''
        Get Number of Points (nop) for sweep

        Input:
            None
        Output:
            nop (int)
        '''
        logging.debug(__name__ + ' : getting Number of Points')
        if self._zerospan:
          return 1
        else:
          if(self.get_cw()):
            self._nop = int(self._visainstrument.ask(':SENS%i:SWE:CW:POIN?' %(self._ci)))
          else:
            self._nop = int(self._visainstrument.ask(':SENS%i:SWE:POIN?' %(self._ci)))
          return self._nop 
    
    def do_set_Average(self, status):
        '''
        Set status of Average

        Input:
            status (boolean)

        Output:
            None
        '''
        logging.debug(__name__ + ' : setting Average to "%s"' % (status))
        if status:
            self._visainstrument.write('SENS%i:AVER:STAT 1' % (self._ci))
            self._visainstrument.write('SENS%i:AVER:TYP SWE' % (self._ci))
        else:
            self._visainstrument.write('SENS%i:AVER:STAT 0' % (self._ci))             
    def do_get_Average(self):
        '''
        Get status of Average

        Input:
            None

        Output:
            Status of Averaging (bool)
        '''
        logging.debug(__name__ + ' : getting average status')
        return bool(int(self._visainstrument.ask('SENS%i:AVER:STAT?' %(self._ci))))
        
        
    
    def do_set_averages(self, av):
        '''
        Set number of averages

        Input:
            av (int) : Number of averages

        Output:
            None
        '''
        if self._zerospan == False:
            logging.debug(__name__ + ' : setting Number of averages to %i ' % (av))
            self._visainstrument.write('SENS%i:AVER:COUN %i' % (self._ci,av))
        else:
            self._visainstrument.write('SWE:POIN %.1f' % (self._ci,av))


    def do_get_averages(self):
        '''
        Get number of averages

        Input:
            None
        Output:
            number of averages
        '''
        logging.debug(__name__ + ' : getting Number of Averages')
        if self._zerospan:
          return int(self._visainstrument.ask('SWE%i:POIN?' % self._ci))
        else:
          return int(self._visainstrument.ask('SENS%i:AVER:COUN?' % self._ci))
                
    def do_set_power(self,pow):
        '''
        Set probe power

        Input:
            pow (float) : Power in dBm

        Output:
            None
        '''
        logging.debug(__name__ + ' : setting power to %s dBm' % pow)
        self._visainstrument.write('SOUR%i:POW:PORT1:LEV:IMM:AMP %.2f' % (self._ci,pow))    
    def do_get_power(self):
        '''
        Get probe power

        Input:
            None

        Output:
            pow (float) : Power in dBm
        '''
        logging.debug(__name__ + ' : getting power')
        return float(self._visainstrument.ask('SOUR%i:POW:PORT1:LEV:IMM:AMP?' % (self._ci)))
                
    def do_set_centerfreq(self,cf):
        '''
        Set the center frequency

        Input:
            cf (float) :Center Frequency in Hz

        Output:
            None
        '''
        logging.debug(__name__ + ' : setting center frequency to %s' % cf)
        self._visainstrument.write('SENS%i:FREQ:CENT %f' % (self._ci,cf))
        self.get_startfreq();
        self.get_stopfreq();
        self.get_span();
    def do_get_centerfreq(self):
        '''
        Get the center frequency

        Input:
            None

        Output:
            cf (float) :Center Frequency in Hz
        '''
        logging.debug(__name__ + ' : getting center frequency')
        return  float(self._visainstrument.ask('SENS%i:FREQ:CENT?'%(self._ci)))
    
    def do_set_cwfreq(self, cf):
        ''' set cw frequency '''
        self._visainstrument.write('SENS%i:FREQ:CW %f' % (self._ci,cf))
    
    def do_get_cwfreq(self):
        ''' get cw frequency '''
        return float(self._visainstrument.ask('SENS%i:FREQ:CW?'%(self._ci)))
    
    def do_set_span(self,span):
        '''
        Set Span

        Input:
            span (float) : Span in KHz

        Output:
            None
        '''
        logging.debug(__name__ + ' : setting span to %s Hz' % span)
        self._visainstrument.write('SENS%i:FREQ:SPAN %i' % (self._ci,span))   
        self.get_startfreq();
        self.get_stopfreq();
        self.get_centerfreq();   
    def do_get_span(self):
        '''
        Get Span
        
        Input:
            None

        Output:
            span (float) : Span in Hz
        '''
        #logging.debug(__name__ + ' : getting center frequency')
        span = self._visainstrument.ask('SENS%i:FREQ:SPAN?' % (self._ci) ) #float( self.ask('SENS1:FREQ:SPAN?'))
        return span

    def do_set_startfreq(self,val):
        '''
        Set Start frequency

        Input:
            span (float) : Frequency in Hz

        Output:
            None
        '''
        logging.debug(__name__ + ' : setting start freq to %s Hz' % val)
        self._visainstrument.write('SENS%i:FREQ:STAR %f' % (self._ci,val))   
        self._start = val
        self.get_centerfreq();
        self.get_stopfreq();
        self.get_span();
    def do_get_startfreq(self):
        '''
        Get Start frequency
        
        Input:
            None

        Output:
            span (float) : Start Frequency in Hz
        '''
        logging.debug(__name__ + ' : getting start frequency')
        self._start = float(self._visainstrument.ask('SENS%i:FREQ:STAR?' % (self._ci)))
        return  self._start

    def do_set_stopfreq(self,val):
        '''
        Set STop frequency

        Input:
            val (float) : Stop Frequency in Hz

        Output:
            None
        '''
        logging.debug(__name__ + ' : setting stop freq to %s Hz' % val)
        self._visainstrument.write('SENS%i:FREQ:STOP %f' % (self._ci,val))  
        self._stop = val
        self.get_startfreq();
        self.get_centerfreq();
        self.get_span();
    def do_get_stopfreq(self):
        '''
        Get Stop frequency
        
        Input:
            None

        Output:
            val (float) : Start Frequency in Hz
        '''
        logging.debug(__name__ + ' : getting stop frequency')
        self._stop = float(self._visainstrument.ask('SENS%i:FREQ:STOP?' %(self._ci) ))
        return  self._stop
        
    def do_get_sweeptime_averages(self):
        if self.get_Average():
            return self.get_sweeptime() * self.get_averages()
        else:
            return self.get_sweeptime()
    
    def do_get_sweeptime(self):
    
        '''
        Get sweep time

        '''
    
        logging.debug(__name__ + ' : getting sweep time')
        
        return float(self.get_nop()) / self.get_bandwidth()

    def do_get_edel(self):   


        '''
        Get electrical delay in seconds for trace 1

        '''
        logging.debug(__name__ + ' : getting electrical delay')
        self._edel = float(self._visainstrument.ask(":CALC1:PAR1:REF:EXT:TIM?"))
        return  self._edel   
        
    
    def do_set_edel(self,val):   

        '''
        Set electrical delay in seconds for trace 1.

        '''
        logging.debug(__name__ + ' : setting electrical delay to %s sec' % val)
        self._visainstrument.write(":CALC1:PAR1:REF:EXT:TIM %e"%(val))
               
               
    def set_edel_auto(self):
        
        '''
        Uses the VNA auto edel function for trace 1 and returns it.
        '''
        
        logging.debug(__name__ + ' : setting electrical delay to auto')
        self._visainstrument.write(":CALC1:PAR1:REF:EXT:AUTO")
        return self.get_edel()
        
    def do_set_bandwidth(self,band):
        '''
        Set Bandwidth

        Input:
            band (float) : Bandwidth in Hz

        Output:
            None
        '''
        logging.debug(__name__ + ' : setting bandwidth to %s Hz' % (band))
        self._visainstrument.write('SENS%i:BWID:RES %i' % (self._ci,band))
    def do_get_bandwidth(self):
        '''
        Get Bandwidth

        Input:
            None

        Output:
            band (float) : Bandwidth in Hz
        '''
        logging.debug(__name__ + ' : getting bandwidth')
        # getting value from instrument
        return  float(self._visainstrument.ask('SENS%i:BWID:RES?'%self._ci))                

    def do_set_cw(self, val):
        '''
        Set instrument to CW (single frequency) mode and back
        '''
        state = 1 if val == True else 0
        self._visainstrument.write(':SENS%i:SWE:CW %d'%(self._ci, state))
        self._cw = val
        
    def do_get_cw(self):
        '''
        retrieve CW mode status from device
        '''
        self._cw = bool(int(self._visainstrument.ask(':SENS%i:SWE:CW?'%(self._ci))))
        return self._cw

    def do_set_zerospan(self,val):
        '''
        Zerospan is a virtual "zerospan" mode. In Zerospan physical span is set to
        the minimal possible value (2Hz) and "averages" number of points is set.

        Input:
            val (bool) : True or False

        Output:
            None
        '''
        #logging.debug(__name__ + ' : setting status to "%s"' % status)
        if val not in [True, False]:
            raise ValueError('set_zerospan(): can only set True or False')        
        if val:
          self._oldnop = self.get_nop()
          self._oldspan = self.get_span()
          if self.get_span() > 0.002:
            Warning('Setting ZVL span to 2Hz for zerospan mode')            
            self.set_span(0.002)
            
        av = self.get_averages()
        self._zerospan = val
        if val:
            self.set_Average(False)
            self.set_averages(av)
            if av<2:
              av = 2
        else: 
          self.set_Average(True)
          self.set_span(self._oldspan)
          self.set_nop(self._oldnop)
          self.get_averages()
        self.get_nop()
               
    def do_get_zerospan(self):
        '''
        Check weather the virtual zerospan mode is turned on

        Input:
            None

        Output:
            val (bool) : True or False
        '''
        return self._zerospan


    def do_set_trigger_source(self,source):
        '''
        Set Trigger Mode

        Input:
            source (string) : AUTO | MANual | EXTernal | REMote

        Output:
            None
        '''
        logging.debug(__name__ + ' : setting trigger source to "%s"' % source)
        if source.upper() in ["AUTO", "MAN", "EXT", "REM"]:
            self._visainstrument.write('TRIG:SOUR %s' % source.upper())        
        else:
            raise ValueError('set_trigger_source(): must be AUTO | MANual | EXTernal | REMote')
    def do_get_trigger_source(self):
        '''
        Get Trigger Mode

        Input:
            None

        Output:
            source (string) : AUTO | MANual | EXTernal | REMote
        '''
        logging.debug(__name__ + ' : getting trigger source')
        return self._visainstrument.ask('TRIG:SOUR?')        
        

    def do_set_channel_index(self,val):
        '''
        Set the index of the channel to address.

        Input:
            val (int) : 1 .. number of active channels (max 16)

        Output:
            None
        '''
        logging.debug(__name__ + ' : setting channel index to "%i"' % int)
        nop = self._visainstrument.read('DISP:COUN?')
        if val < nop:
            self._ci = val 
        else:
            raise ValueError('set_channel_index(): index must be < nop channels')
    
    def do_get_channel_index(self):
        '''
        Get active channel

        Input:
            None

        Output:
            channel_index (int) : 1-16
        '''
        logging.debug(__name__ + ' : getting channel index')
        return self._ci
          
    def do_get_source_attenuation(self,channel):
        '''
        Get status of source attenuation on port 

        Input:
            None

        Output:
            Status of source attenuation (value in dB) 
        '''
        
        logging.debug(__name__ + ' : getting attenuation status')
        return int(self._visainstrument.ask('SOUR%i:POW:PORT%i:ATT?' %(self._ci,channel)))
    
    def do_set_source_attenuation(self, att,channel):
        '''
        Set value of source attenuation

        Input:
            att (int)   :   Value of source attenuation must
                            must be one of (0,10,20,30,40,50,60)

        Output:
            None
        '''
        if att in (0,10,20,30,40,50,60):
            logging.debug(__name__ + ' : setting attenuation on port %i to %idB ' % (channel,att))
            self._visainstrument.write('SOUR%i:POW:PORT%i:ATT %i' % (self._ci,channel,att))
        else:
            logging.debug(__name__ + ' : cannot set attenuation to %i ' % (att))
    
    def do_get_source_power_start(self,channel):
        '''
        Get starting value for power sweep 

        Input:
            None

        Output:
            Starting value for the power sweep (float value in dBm) 
        '''
        
        logging.debug(__name__ + ' : getting starting power')
        return float(self._visainstrument.ask('SOUR%i:POW:PORT%i:LIN:POW:STAR?' %(self._ci,channel)))
                    
    def do_set_source_power_start(self, pow,channel):
        '''
        Set starting value for power sweep 

        Input:
            pow (float)   :   Starting power (-29.9 to 30)

        Output:
            None
        '''
        
        logging.debug(__name__ + ' : setting starting power on port %i to %.2f dBm ' % (channel,pow))
        self._visainstrument.write('SOUR%i:POW:PORT%i:LIN:POW:STAR %.2f' % (self._ci,channel,pow))
        
    def do_get_source_power_stop(self,channel):
        '''
        Get stopping value for power sweep 

        Input:
            None

        Output:
            Stopping value for the power sweep (float value in dBm) 
        '''
        
        logging.debug(__name__ + ' : getting stopping power')
        return float(self._visainstrument.ask('SOUR%i:POW:PORT%i:LIN:POW:STOP?' %(self._ci,channel)))
                    
    def do_set_source_power_stop(self, pow,channel):
        '''
        Set stopping value for power sweep 

        Input:
            pow (float)   :   Stopping power (-29.9 to 30)

        Output:
            None
        '''
        
        logging.debug(__name__ + ' : setting stopping power on port %i to %.2f dBm ' % (channel,pow))
        self._visainstrument.write('SOUR%i:POW:PORT%i:LIN:POW:STOP %.2f' % (self._ci,channel,pow))
        
      
    def do_set_calibration_state(self, status):
        '''
        Set status of Calibration

        Input:
            status (boolean) 

        Output:
            None
        '''
        
        if status:
            logging.debug(__name__ + ' : Turning on calibration ')
            self._visainstrument.write('SENS%i:CORR:STAT ON' % (self._ci))
        else:
            logging.debug(__name__ + ' : Turning off calibration ')
            self._visainstrument.write('SENS%i:CORR:STAT OFF' % (self._ci))
        
    def do_get_calibration_state(self):
        '''
        Get status of Calibration

        Input:
            None

        Output:
            Status of Calibration (bool)
        '''
        logging.debug(__name__ + ' : getting calibration state status')
        return bool(int(self._visainstrument.ask('SENS%i:CORR:STAT?' %(self._ci))))
    
    def do_get_sweep_type(self):
        '''
        Get the Sweep Type

        Input:
            None

        Output:
            Sweep Type (string). One of
            LIN:    Frequency-based linear sweep
            LOG:    Frequency-based logarithmic sweep
            FSEG:   Segment-based sweep with frequency-based segments
            ISEG:   Index-based sweep with frequency-based segments
            POW:    Power-based sweep with either Power-based sweep with a 
                CW frequency, or Power-based sweep with swept-frequency
            MFGC:   Multiple frequency gain compression
        '''
        logging.debug(__name__ + ' : getting sweep type')
        
        return str(self._visainstrument.ask('SENS%i:SWE:TYP?' %(self._ci)))
    
    def do_set_sweep_type(self,swtype):
        '''
        Set the Sweep Type
        Input:
            swtype (string):    One of
                LIN:    Frequency-based linear sweep
                LOG:    Frequency-based logarithmic sweep
                FSEG:   Segment-based sweep with frequency-based segments
                ISEG:   Index-based sweep with frequency-based segments
                POW:    Power-based sweep with either Power-based sweep with a 
                    CW frequency, or Power-based sweep with swept-frequency
                MFGC:   Multiple frequency gain compression

        Output:
            None            
        '''
        if swtype in ('LIN','LOG','FSEG','ISEG','POW','MFGC'):
            
            logging.debug(__name__ + ' : Setting sweep type to %s'%(swtype))
            return self._visainstrument.write('SENS%i:SWE:TYP %s' %(self._ci,swtype))
            
        else:
            logging.debug(__name__ + ' : Illegal argument %s'%(swtype))
            

    def do_get_power_nop(self,channel):
        '''
        Get number of points for a power sweep 

        Input:
            None

        Output:
            Number of points for a power sweep (int) 
        '''
        
        logging.debug(__name__ + ' : getting power nop')
        return int(self._visainstrument.ask(':SOUR%i:POW:PORT%i:LIN:POW:POIN?' %(self._ci,channel)))
    



    def do_set_power_nop(self, n,channel):
        '''
        Set number of points for a power sweep 

        Input:
            n (int)   :   Number of points of the sweep

        Output:
            None
        '''
        
        logging.debug(__name__ + ' : setting number of power sweep points on channel %i to %i ' % (channel,n))
        self._visainstrument.write(':SOUR%i:POW:PORT%i:LIN:POW:POIN %i' % (self._ci,channel,n))
        
    def do_get_avg_type(self):
        '''
        Get the Averaging Type

        Input:
            None

        Output:
            Averaging Type (string). One of
            POIN:   Point by Point
            SWE:    Sweep by Sweep
            
        '''
        logging.debug(__name__ + ' : getting averaging type')
        
        return str(self._visainstrument.ask('SENS%i:AVER:TYP?' %(self._ci)))
    
    def do_set_avg_type(self,avgtype):
        '''
        Set the Averaging Type
        Input:
            avgtype (string). One of:
                POIN:   Point by Point
                SWE:    Sweep by Sweep

        Output:
            None            
        '''
        if avgtype in ('POIN','SWE'):
            
            logging.debug(__name__ + ' : Setting avg type to %s'%(avgtype))
            return self._visainstrument.write('SENS%i:AVER:TYP %s' %(self._ci,avgtype))
            
        else:
            logging.debug(__name__ + ' : Illegal argument %s'%(avgtype))


    def read(self, msg):
        return self._visainstrument.read(msg)
    def write(self,msg):
        return self._visainstrument.write(msg)    
    def ask(self,msg):
        return self._visainstrument.ask(msg)
    
    def pre_measurement(self):
        '''
        Set everything needed for the measurement
        Averaging has to be enabled.
        '''
        if not self.get_Average():
            self.set_Average(True)
            self.set_averages(1)
        
    def post_measurement(self):
        '''
        Bring the VNA back to a mode where it can be easily used by the operater.
        For this VNA and measurement method, no special things have to be done.
        '''
        pass
        
    def start_measurement(self):
        '''
        This function is called at the beginning of each single measurement in the spectroscopy script.
        Here, it resets the averaging.  
        '''
        self.avg_clear()
    
    def ready(self):
        '''
        This is a proxy function, returning True when the VNA has finished the required number of averages.
        '''
        return self.avg_status() == self.get_averages(query=False)
            
            



import qkit
from qkit.core.instrument_base import Instrument
import numpy as np
from time import sleep

class virtual_yokomagnet(Instrument):

    def __init__(self, name, source=None, channel = 0, stepsize = 2e-9, stepdelay=1):
        '''
            virtual magnet built upon yokogawa source
        '''
        Instrument.__init__(self, name, tags=['virtual'])
        self._instruments = qkit.instruments.get_instruments()
        self._source= self._instruments.get(source)
        self._channel = channel        
        # Add parameters        
        self.add_parameter('current', type=float,
            flags=Instrument.FLAG_GETSET, units='A')
        self.add_parameter('stepsize', type=float,
            flags=Instrument.FLAG_GETSET, units='A')        
        self.add_parameter('range', type=float,
            flags=Instrument.FLAG_GETSET, units='A')        
        self.add_parameter('stepdelay', type=float,
            flags=Instrument.FLAG_GETSET, units='ms')

        self._stepsize = stepsize
        self._stepdelay = stepdelay
        getattr(self._source, 'set_ch%d_sweep_mode'%channel)('fix')
        getattr(self._source, 'set_ch%d_source_mode'%channel)('curr')
        getattr(self._source, 'set_ch%d_sense_mode'%channel)('curr')

        self.get_all()

    def get_all(self):
        self.get_current()
        self.get_stepsize()
        self.get_stepdelay()
        self.get_range()

    def do_get_current(self):
        self._old = getattr(self._source, 'get_ch%d_level'%(self._channel))()
        return self._old

    def do_set_current( self, cur):
        if np.abs(cur) > self._range:
            raise ValueError('set_current(): given value exeeds the range of %f' % self._range)
        old = self._old
        stepsize =  self.get_stepsize()
        stepdelay = self.get_stepdelay()

        if np.abs(old-cur) <= stepsize:
            steps = np.array([])
        elif old > cur:				
            steps = np.arange(old,cur-stepsize,-stepsize)
        else:
            steps = np.arange(old,cur+stepsize,stepsize)
        steps = np.append(steps, cur)

        for step in steps:
            getattr(self._source, 'set_ch%d_level'%(self._channel))(step)
            sleep(stepdelay*1e-3)
        self._old = cur #Putting new value to the buffer		
        
    def do_get_stepsize(self):
        return self._stepsize
    def do_set_stepsize(self, val):
        self._stepsize = val
        
    def do_get_stepdelay(self):
        return self._stepdelay
    def do_set_stepdelay(self, val):
        self._stepdelay = val
        
    def do_get_range(self):
        self._range = getattr(self._source, 'get_ch%d_source_range'%self._channel)()
        return self._range
    def do_set_range(self, val):
        getattr(self._source, 'set_ch%d_source_range'%self._channel)(val)
        self._do_get_range()



from qkit.core.instrument_base import Instrument
import types

class manual_settings(Instrument):

    def __init__(self, name):
        Instrument.__init__(self, name, tags=['virtual'])
        self.add_function('add_manual')

    def add_manual(self, name, type=float, **kwargs):
        self.add_parameter(name, type=type,
                flags=Instrument.FLAG_SET|Instrument.FLAG_SOFTGET,
                set_func=lambda x: True,
                **kwargs)




# worker class for parameter value readout written by HR,JB@KIT 2016

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import time

from threading import Thread 
        
class IO_worker(Thread):
    '''
    Worker class which triggers the acquisition of all parameters from the instruments via the server. 
    Inherits from thread.
    '''
    def __init__(self,DATA):
        self.DATA = DATA
        Thread.__init__(self)
        
    def run(self):
        '''
        Run method of the worker thread.
        '''
        print('Worker started.')
        while True:
            for p in self.DATA.parameters:
                if p.logging:
                    p.create_logfile()
            while int(time.strftime("%H%M",time.localtime(time.time()+10))) != 0:
                #readout all parameters
                for p in self.DATA.parameters:
                    if p.schedule():
                        p.store_value(p.data_request_object())
                        if self.DATA.debug: print('readout %s.'%p.name)
                                
                time.sleep(self.DATA.cycle_time)
            for p in self.DATA.parameters:
                if p.logging: p.close_logfile()
            time.sleep(70)





#
# MicroMag3+mbed 3d fluxgate magnetometer
#

from qkit.core.instrument_base import Instrument
from time import time, sleep
from qkit import visa
import types
import logging

class MicroMag3(Instrument):
    '''
    This driver communicates with Markus's custom fluxgate magnetometer setup.

    Usage:
    Initialize with
    <name> = instruments.create('name', 'MicroMag3', address='<Instrument address>')
    <Instrument address> = ASRL3::INSTR

    '''

    def __init__(self, name, address):
        '''

        Input:
            name (string)    : name of the instrument
            address (string) : instrument (serial port) address

        Output:
            None
        '''
        logging.debug(__name__ + ' : Initializing instrument')
        Instrument.__init__(self, name, tags=['physical'])


        self._address = address
        self._visainstrument = visa.SerialInstrument(self._address, delay = 30e-3)
        self._values = {}
        self._visainstrument.stop_bits = 1  # This works only using pyvisa 1.4
        self.reset()

        #Add parameters
        self.add_parameter('b_raw', type=int,
            flags=Instrument.FLAG_GET)
        self.add_parameter('counts', type=int,
            flags=Instrument.FLAG_GETSET)

        # Add functions
        self.add_function('get_all')
        self.get_all()

    def get_all(self):
        '''
        Reads all implemented parameters from the instrument,
        and updates the wrapper.

        Input:
            None

        Output:
            None
        '''
        logging.info(__name__ + ' : reading all settings from instrument')
        self.reset()
        self.get_b_raw()
        self.get_counts()


    # Functions
    def _execute(self, message):
        '''
        Write a command to the device

        Input:
            message (str) : write command for the device

        Output:
            None
        '''
        logging.info(__name__ + ' : Send the following command to the device: %s' % message)
        try:
            result = self._visainstrument.ask('@%s%s' % (self._number, message))
        except:
            self.reset()
            
        if result.find('?') >= 0:
            print "Error: Command %s not recognized" % message
        else:
            return result

    def reset(self):
        '''
        Clear serial input buffer from stale values
        '''
        self._visainstrument.clear()

    def do_get_b_raw(self):
        '''
            get uncalibrated magnetic field
        '''
        raw_mag_str = self._visainstrument.ask(':MAG:RAW?')
        raw_mag_vect = map(int, raw_mag_str.split(" "))
        return raw_mag_vect

    def do_set_counts(self, value):
        '''
            set number of measurement cycles
            number must be 2^n, n = 5..13
            larger count results in longer measurement time, higher accuracy and lower maximum field
        '''
        if not value in [32, 64, 128, 256, 512, 1024, 2048, 4096]:
            raise ValueError('Number of counts must be a power of 2 between 32 and 4096.')
        self._visainstrument.write(':MAG:COUNT %d'%value)

    def do_get_counts(self):
        '''
            get number of measurement cycles
        '''
        return self._visainstrument.ask(':MAG:COUNT?')



# Agilent_VNA_E5071C driver, P. Macha, modified by M. Weides July 2013, J. Braumueller 2015
# Adapted to Keysight VNA by A. Schneider and L. Gurenhaupt 2016
# Updated from E5071B to E5080B by M. Wildermuth and M. Kristen 2019
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, query to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import qkit
from qkit.core.instrument_base import Instrument
from qkit import visa
import logging
import numpy

class Keysight_VNA_E5080B(Instrument):
    """
    This is the python driver for the Anritsu MS4642A Vector Network Analyzer

    Usage:
    Initialise with
    <name> = instruments.create('<name>', address='<GPIB address>', reset=<bool>)

    """

    def __init__(self, name, address, channel_index=1):
        """
        Initializes

        Input:
            name (string)    : name of the instrument
            address (string) : GPIB address
        """
        
        logging.info(__name__ + ' : Initializing instrument')
        Instrument.__init__(self, name, tags=['physical'])

        self._address = address
        self.reconnect()
        self._freqpoints = 0
        self._ci = channel_index 
        self._pi = 2 # port_index, similar to self._ci
        self._start = 0
        self._stop = 0
        self._nop = 0
        self._cwfreq = None #MK This parameter tracks the center-freq when switching to nop=1 (cw mode)
        self._hold = None
        self._sweep = None
        self._edel = None
        self._active_trace = None

        # Implement parameters
        self.add_parameter('nop', type=int,
            flags=Instrument.FLAG_GETSET,
            minval=1, maxval=100001,
            tags=['sweep'])

        self.add_parameter('bandwidth', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=1e9,
            units='Hz', tags=['sweep']) 

        self.add_parameter('averages', type=int,
            flags=Instrument.FLAG_GETSET,
            minval=1, maxval=65536, tags=['sweep'])

        self.add_parameter('count', type=int,
            flags=Instrument.FLAG_GETSET,
            minval=1, maxval=2e6, tags=['sweep'])

        self.add_parameter('Average', type=bool,
            flags=Instrument.FLAG_GETSET)   
                    
        self.add_parameter('centerfreq', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=20e9,
            units='Hz', tags=['sweep'])

        self.add_parameter('cwfreq', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=20e9,
            units='Hz', tags=['sweep'])
            
        self.add_parameter('startfreq', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=20e9,
            units='Hz', tags=['sweep'])            
            
        self.add_parameter('stopfreq', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=20e9,
            units='Hz', tags=['sweep'])                        
            
        self.add_parameter('span', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=20e9,
            units='Hz', tags=['sweep'])        
            
        self.add_parameter('power', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=-100, maxval=20, offset=True,
            units='dBm', tags=['sweep'])

        self.add_parameter('startpower', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=-100, maxval=20,
            units='dBm')

        self.add_parameter('stoppower', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=-100, maxval=20,
            units='dBm')

        self.add_parameter('cw', type=bool,
            flags=Instrument.FLAG_GETSET)
            
        self.add_parameter('channel_index', type=int,
            flags=Instrument.FLAG_GETSET)

        self.add_parameter('sweeptime', type=float,   #added by MW
            flags=Instrument.FLAG_GET,
            minval=0, maxval=1e3,
            units='s', tags=['sweep'])
            
        self.add_parameter('sweeptime_averages', type=float,   #JB
            flags=Instrument.FLAG_GET,
            minval=0, maxval=1e3,
            units='s', tags=['sweep'])
        
        self.add_parameter('measurement_parameter', type=str,
                           flags=Instrument.FLAG_GETSET)
    
        self.add_parameter('edel', type=float, # legacy name for parameter. This corresponds to the VNA's port extension values.
            flags=Instrument.FLAG_GETSET, 
            minval=-10, maxval=10,
            units='s', tags=['sweep'],
            channels=(1, self._pi), channel_prefix = 'port%d_') # the channel option for qtlab's Instument class is used here to easily address the two VNA ports

        self.add_parameter('edel_status', type=bool, # legacy name for parameter. This corresponds to the VNA's port extension values.
            flags=Instrument.FLAG_GETSET)
                   
        self.add_parameter('sweep_mode', type=str,  #JDB This parameter switches on/off hold. The hold function below does the same job, this is just for code compatibility to the agilent and anritsu drivers.
            flags=Instrument.FLAG_GETSET,tags=['sweep']) 
                    
        self.add_parameter('sweep_type', type=str,
            flags=Instrument.FLAG_GETSET,tags=['sweep'])

        self.add_parameter('active_trace', type=int,
            flags=Instrument.FLAG_GETSET)
                    
        #Triggering Stuff
        self.add_parameter('trigger_source', type=str,
            flags=Instrument.FLAG_GETSET)
        
        # Implement functions
        self.add_function('get_freqpoints')
        self.add_function('get_tracedata')
        self.add_function('avg_clear')
        self.add_function('avg_status')
        self.add_function('get_hold')
        self.add_function('hold')
        self.add_function('get_sweeptime')
        self.add_function('get_sweeptime_averages')
        self.add_function('pre_measurement')
        self.add_function('start_measurement')
        self.add_function('ready')
        self.add_function('post_measurement')

        self.do_set_active_trace(1)
        self.get_all()
    
    def get_all(self):        
        self.get_nop()
        self.get_power()
        self.get_centerfreq()
        self.get_startfreq()
        self.get_stopfreq()
        self.get_span()
        self.get_bandwidth()
        self.get_trigger_source()
        self.get_Average()
        self.get_averages()
        self.get_freqpoints()   
        self.get_channel_index()
        self.get_sweeptime()
        self.get_sweeptime_averages()
        self.get_edel_status()
        self.get_cw()
        for port in range(self._pi):
            self.get('port%d_edel' % (port+1))
        
    ###
    #Communication with device
    ###
    def reset_vna(self):
        """
        Deletes all traces, measurements, and windows and resets the vna to factory
        defined default settings.
        Creates a S11 measurement named "CH1_S11_1".
        """
        self._visainstrument.write('SYST:PRES')
    
    def hold(self, status):
        if status:
            self._visainstrument.write('SENS%i:SWE:MODE HOLD'%self._ci)
        else:
            self._visainstrument.write('SENS%i:SWE:MODE CONT'%self._ci)

    def get_hold(self):
        self._hold=str(self._visainstrument.query('SENS%i:SWE:MODE?'%self._ci)).rstrip()
        return self._hold == 'HOLD'
        
    def avg_clear(self):
        self._visainstrument.write(':SENS%i:AVER:CLE' % self._ci)

    def avg_status(self):
        return 0 == (int(self._visainstrument.query('STAT:OPER:COND?')) & (1<<4))

    def get_tracedata(self, format='AmpPha', single=False, averages=None):
        """
        Get the data of the current trace

        Input:
            format (string) : 'AmpPha': Amp in dB and Phase, 'RealImag',

        Output:
            'AmpPha':_ Amplitude and Phase
        """
        if single:
            print('No longer implemented. Use manual trigger source.')
        if averages is not None:

            print('Average parameter no longer supported.')

        self._visainstrument.write('FORM:DATA REAL,32')
        self._visainstrument.write('FORM:BORD SWAPPED') #SWAPPED
        data = self._visainstrument.query_binary_values('CALC%i:MEAS%i:DATA:SDAT?' %( self._ci,self._active_trace))
        data_size = numpy.size(data)
        datareal = numpy.array(data[0:data_size:2])
        dataimag = numpy.array(data[1:data_size:2])
          
        if format == 'RealImag':
            if self.get_cw():
                return numpy.mean(datareal), numpy.mean(dataimag)
            else:
                return datareal, dataimag
        elif format == 'AmpPha':
            if self.get_cw():
                datacomplex = [numpy.mean(datareal + 1j*dataimag)]
                dataamp = numpy.abs(datacomplex)
                datapha = numpy.angle(datacomplex)
            else:
                dataamp = numpy.sqrt(datareal*datareal+dataimag*dataimag)
                datapha = numpy.arctan2(dataimag, datareal)
            return dataamp, datapha
        else:
            raise ValueError('get_tracedata(): Format must be AmpPha or RealImag')
    
    def get_segments(self):
        if self.get_sweep_type(query=False) == "SEGM":
            self._visainstrument.write('FORM:DATA REAL,64')
            self._visainstrument.write('FORM:BORD SWAPPED')
            segments =  [0]
            for x in numpy.reshape(self._visainstrument.query_binary_values("sense:segment:list? SSTOP",datatype="d"),(-1,8)):
                if x[0]>0.5: #the segment is active
                    segments.append(int(x[1])+segments[-1])
            return segments[1:]
        else:
            return []
      
    def get_freqpoints(self, query=False):
        if self.get_sweep_type(query=False) == "SEGM":
            self._visainstrument.write('FORM:DATA REAL,64')
            self._visainstrument.write('FORM:BORD SWAPPED')
            freqs = numpy.array([])
            for x in numpy.reshape(self._visainstrument.query_binary_values("sense:segment:list? SSTOP",datatype="d"),(-1,8)):
                if x[0]>0.5: #the segment is active
                    freqs = numpy.append(freqs,numpy.linspace(x[2],x[3],int(x[1])))
            self._freqpoints = freqs
            return self._freqpoints
        if query:
            self._freqpoints = numpy.array(self._visainstrument.query_ascii_values('SENS:X?'))
        if self.get_cw():
            self._freqpoints = numpy.atleast_1d(self.get_centerfreq())
        else:
            self._freqpoints = numpy.linspace(self._start,self._stop,self._nop)
        return self._freqpoints

    ###
    # SET and GET functions
    ###
    
    def do_set_sweep_mode(self, mode):
        """
        select the sweep mode from 'hold', 'cont', single' and "group"
        single means only one single trace, not all the averages even if averages
         larger than 1 and Average==True
        """
        mode=mode.lower()
        if mode == 'hold':
            self._visainstrument.write('SENS%i:SWE:MODE HOLD' % self._ci)
        elif mode == 'cont':
            self._visainstrument.write('SENS%i:SWE:MODE CONT' % self._ci)
        elif mode == 'single':
            self._visainstrument.write('SENS%i:SWE:MODE SING' % self._ci)
        elif mode == 'group':
            self._visainstrument.write('SENS%i:SWE:MODE GRO' % self._ci)
        else:
            logging.warning('invalid mode')
            
    def do_get_sweep_mode(self):
        return str(self._visainstrument.query(':SENS%i:SWE:MODE?' % self._ci)).rstrip()
    
    def do_set_nop(self, nop):
        """
        Set Number of Points (nop) for sweep

        Input:
            nop (int) : Number of Points

        Output:
            None
        """
        logging.debug(__name__ + ' : setting Number of Points to %s ' % nop)
        self._visainstrument.write(':SENS%i:SWE:POIN %i' %(self._ci,nop))
        self._nop = nop
        self.get_freqpoints() #Update List of frequency points
        
    def do_get_nop(self):
        """
        Get Number of Points (nop) for sweep

        Input:
            None
        Output:
            nop (int)
        """

        self._nop = int(self._visainstrument.query(':SENS%i:SWE:POIN?' % self._ci))
        return self._nop 
    
    def do_set_Average(self, status):
        """
        Set status of Average

        Input:
            status (boolean)

        Output:
            None
        """
        logging.debug(__name__ + ' : setting Average to "%s"' % status)
        if status:
            status = 'ON'
            self._visainstrument.write('SENS%i:AVER:STAT %s' % (self._ci,status))
        elif not status:
            status = 'OFF'
            self._visainstrument.write('SENS%i:AVER:STAT %s' % (self._ci,status))
        else:
            raise ValueError('set_Average(): can only set True or False')

    def do_get_Average(self):
        """
        Get status of Average

        Input:
            None

        Output:
            Status of Averaging (boolean)
        """
        logging.debug(__name__ + ' : getting average status')
        return bool(int(self._visainstrument.query('SENS%i:AVER:STAT?' % self._ci)))
                    
    def do_set_averages(self, av):
        """
        Set number of averages

        Input:
            av (int) : Number of averages
        Output:
            None
        """
        logging.debug(__name__ + ' : setting Number of averages to %i ' % av)
        self._visainstrument.write('SENS%i:AVER:COUN %i' % (self._ci,av))
        self._visainstrument.write(':SENS%i:AVER:CLE' % self._ci)
            
    def do_get_averages(self):
        """
        Get number of averages

        Input:
            None
        Output:
            Number of averages
        """
        logging.debug(__name__ + ' : getting Number of Averages')
        return int(self._visainstrument.query('SENS%i:AVER:COUN?' % self._ci))

    def do_set_count(self, co):
        """
        Sets the trigger count (groups)

        Input:
            co (int) : Count number
        Output:
            None
        """
        logging.debug(__name__ + ' : setting count number to %i ' % co)
        self._visainstrument.write('SENS%i:SWE:GRO:COUN %i' % (self._ci,co))

    def do_get_count(self):
        """
        Sets the trigger count (groups)

        Input:
            None
        Output:
            Count number
        """
        logging.debug(__name__ + ' : getting count number')
        return int(self._visainstrument.query('SENS%i:SWE:GRO:COUN?' % self._ci))

    def do_set_power(self, pow, port=1):
        """
        Set probe power

        Input:
            pow (float) : Power in dBm

        Output:
            None
        """
        logging.debug(__name__ + ' : setting power to %s dBm' % pow)
        self._visainstrument.write('SOUR%i:POW%i %.1f' % (self._ci, port, pow))

    def do_get_power(self, port=1):
        """
        Get probe power

        Input:
            None

        Output:
            pow (float) : Power in dBm
        """
        logging.debug(__name__ + ' : getting power')
        return float(self._visainstrument.query('SOUR%i:POW%i?' % (self._ci, port)))

    def do_set_startpower(self, pow):
        """
        Set probe power

        Input:
            pow (float) : Power in dBm

        Output:
            None
        """
        logging.debug(__name__ + ' : setting startpower to %s dBm' % pow)
        self._visainstrument.write('SOUR%i:POW:START %.1f' % (self._ci,pow))

    def do_get_startpower(self):
        """
        Get probe power

        Input:
            None

        Output:
            pow (float) : Power in dBm
        """
        logging.debug(__name__ + ' : getting startpower')
        return float(self._visainstrument.query('SOUR%i:POW:START?' % self._ci))

    def do_set_stoppower(self, pow):
        """
        Set probe power

        Input:
            pow (float) : Power in dBm

        Output:
            None
        """
        logging.debug(__name__ + ' : setting stoppower to %s dBm' % pow)
        self._visainstrument.write('SOUR%i:POW:STOP %.1f' % (self._ci,pow))

    def do_get_stoppower(self):
        """
        Get probe power

        Input:
            None

        Output:
            pow (float) : Power in dBm
        """
        logging.debug(__name__ + ' : getting stoppower')
        return float(self._visainstrument.query('SOUR%i:POW:STOP?' % self._ci))
                
    def do_set_centerfreq(self, cf):
        """
        Set the center frequency

        Input:
            cf (float) :Center Frequency in Hz

        Output:
            None
        """
        logging.debug(__name__ + ' : setting center frequency to %s' % cf)
        self._visainstrument.write('SENS%i:FREQ:CENT %f' % (self._ci,cf))
        self.get_startfreq()
        self.get_stopfreq()
        self.get_span()

    def do_get_centerfreq(self):
        """
        Get the center frequency

        Input:
            None

        Output:
            cf (float) :Center Frequency in Hz
        """
        logging.debug(__name__ + ' : getting center frequency')
        return  float(self._visainstrument.query('SENS%i:FREQ:CENT?' % self._ci))

    def do_set_cwfreq(self, cf):
        """ set cw frequency """
        if self.get_cw():
            self._visainstrument.write('SENS%i:FREQ:CENT %f' %(self._ci, cf))
            self._cwfreq = cf
        else:
            self._cwfreq = cf

    def do_get_cwfreq(self):
        """ get cw frequency """
        if self.get_cw():
            return  float(self._visainstrument.query('SENS%i:FREQ:CENT?' % self._ci))
        else:
            return self._cwfreq

    def do_set_span(self, span):
        """
        Set Span

        Input:
            span (float) : Span in KHz

        Output:
            None
        """
        logging.debug(__name__ + ' : setting span to %s Hz' % span)
        self._visainstrument.write('SENS%i:FREQ:SPAN %i' % (self._ci,span))
        self.get_startfreq()
        self.get_stopfreq()
        self.get_centerfreq()
        
    def do_get_span(self):
        """
        Get Span

        Input:
            None

        Output:
            span (float) : Span in Hz
        """
        span = self._visainstrument.query('SENS%i:FREQ:SPAN?' % self._ci) #float( self.query('SENS1:FREQ:SPAN?'))
        return span

    def do_get_sweeptime_averages(self):###JB
        """
        Get sweeptime

        Input:
            None

        Output:
            sweep time (float) times number of averages: sec
        """
        return self.get_sweeptime() * (self.get_averages() if self.get_Average() else 1)
        
    def do_get_sweeptime(self):  #added MW July 2013
        """
        Get sweeptime

        Input:
            None

        Output:
            sweep time (float) : sec
        """
        logging.debug(__name__ + ' : getting sweep time')
        self._sweep=float(self._visainstrument.query('SENS1:SWE:TIME?'))
        return self._sweep

    def do_set_edel(self, val, channel):  # MP 04/2017
        """
        Set electrical delay
        """
        logging.debug(__name__ + ' : setting port %s extension to %s sec' % (channel, val))
        self._visainstrument.write('SENS1:CORR:EXT:PORT%i:TIME %.12f' % (channel, val))

    def do_get_edel(self, channel):   # MP 04/2017
        """
        Get electrical delay
        """
        logging.debug(__name__ + ' : getting port %s extension' % channel)
        self._edel = float(self._visainstrument.query('SENS1:CORR:EXT:PORT%i:TIME?'% channel))
        return  self._edel   
        
    def do_set_edel_status(self, status):   # AS 04/2019
        """
        Set electrical delay
        """
        logging.debug(__name__ + ' : setting port extension status to %s' % status)
        self._visainstrument.write('SENS:CORR:EXT:STAT %i' % status)

    def do_get_edel_status(self):   # AS 04/2019

        """
        Get electrical delay

        """
        logging.debug(__name__ + ' :  port extension status')
        return  self._visainstrument.query('SENS:CORR:EXT:STAT?').strip() == "1"

    def do_set_startfreq(self, val):
        """
        Set Start frequency

        Input:
            span (float) : Frequency in Hz

        Output:
            None
        """
        logging.debug(__name__ + ' : setting start freq to %s Hz' % val)
        self._visainstrument.write('SENS%i:FREQ:STAR %f' % (self._ci,val))
        self._start = val
        self.get_centerfreq()
        self.get_stopfreq()
        self.get_span()
        
    def do_get_startfreq(self):
        """
        Get Start frequency

        Input:
            None

        Output:
            span (float) : Start Frequency in Hz
        """
        logging.debug(__name__ + ' : getting start frequency')
        self._start = float(self._visainstrument.query('SENS%i:FREQ:STAR?' % self._ci))
        return  self._start

    def do_set_stopfreq(self, val):
        """
        Set STop frequency

        Input:
            val (float) : Stop Frequency in Hz

        Output:
            None
        """
        logging.debug(__name__ + ' : setting stop freq to %s Hz' % val)
        self._visainstrument.write('SENS%i:FREQ:STOP %f' % (self._ci,val))
        self._stop = val
        self.get_startfreq()
        self.get_centerfreq()
        self.get_span()

    def do_get_stopfreq(self):
        """
        Get Stop frequency

        Input:
            None

        Output:
            val (float) : Start Frequency in Hz
        """
        logging.debug(__name__ + ' : getting stop frequency')
        self._stop = float(self._visainstrument.query('SENS%i:FREQ:STOP?' % self._ci))
        return  self._stop

    def do_set_bandwidth(self, band):
        """
        Set Bandwidth

        Input:
            band (float) : Bandwidth in Hz

        Output:
            None
        """
        logging.debug(__name__ + ' : setting bandwidth to %s Hz' % band)
        self._visainstrument.write('SENS%i:BWID %i' % (self._ci,band))

    def do_get_bandwidth(self):
        """
        Get Bandwidth

        Input:
            None

        Output:
            band (float) : Bandwidth in Hz
        """
        logging.debug(__name__ + ' : getting bandwidth')
        # getting value from instrument
        return  float(self._visainstrument.query('SENS%i:BWID?'%self._ci))
    
    def do_set_cw(self, val):
        """
        Set instrument to CW (single frequency) mode and back
        """
        if val:
            if self._cwfreq is None:
                self._cwfreq = self.get_centerfreq()
            self.set_nop(1)
            self.set_sweep_type("LIN")
            self.set_cwfreq(self._cwfreq)
        else:
            self._visainstrument.write(':SENS%i:SWE:POIN 1001' % self._ci)
            self.set_startfreq(self._start)
            self.set_stopfreq(self._stop)

    def do_get_cw(self):
        """
        retrieve CW mode status
        """
        if (self.get_nop() == 1) and (self.get_sweep_type() == "LIN"): ret = True
        else: ret = False
        return ret

    def do_set_channel_index(self, val):
        """
        Set the index of the channel to address.

        Input:
            val (int) : 1 .. number of active channels (max 14)

        Output:
            None
        """
        logging.debug(__name__ + ' : setting channel index to "%i"' % val)
        noc = int(self._visainstrument.query('SYST:CHAN:CAT?')[-3])
        if val <= noc:
            self._ci = val 
        else:
            raise ValueError('set_channel_index(): index must be < No. channels')

    def do_get_channel_index(self):
        """
        Get the index of the channel to address.

        Input:
            None

        Output:
            channel_index (int) : 1-14
        """
        logging.debug(__name__ + ' : getting channel index')
        return self._ci

    def do_set_active_trace(self, trace):
        """
        Sets the active trace on the selected channel, which can then be readout
        :param trace: Number of the active trace
        :return: None
        """
        # TODO: catch error
        self._active_trace = trace
        self._visainstrument.write('CALC%i:PAR:MNUM %f' %(self._ci, trace))

    def do_get_active_trace(self):
        """
        :return: the active trace on the selected channel
        """
        self._active_trace = int(self._visainstrument.query('CALC%i:PAR:MNUM?'%(self._ci)).strip())
        return self._active_trace

    def do_get_sweep_type(self):
        """
        Get the Sweep Type

        Input:
            None

        Output:
            Sweep Type (string). One of
            LIN:    Frequency-based linear sweep
            LOG:    Frequency-based logarithmic sweep
            SEGM:   Segment-based sweep with frequency-based segments
            POW:    Power-based sweep with CW frequency
            CW:     Single frequency mode
        """
        logging.debug(__name__ + ' : getting sweep type')
        return str(self._visainstrument.query('SENS%i:SWE:TYPE?' % self._ci)).rstrip()
    
    def do_set_sweep_type(self, swtype):
        """
        Set the Sweep Type
        Input:
            swtype (string):    One of
                LIN:    Frequency-based linear sweep
                LOG:    Frequency-based logarithmic sweep
                SEGM:   Segment-based sweep with frequency-based segments
                POW:    Power-based sweep with CW frequency
                CW:     Time-based sweep with CW frequency

        Output:
            None
        """
        if swtype in ('LIN','LOG','SEGM','POW','CW'):
            logging.debug(__name__ + ' : Setting sweep type to %s' % swtype)
            if swtype == 'SEGM':
                self._visainstrument.write('SENS%i:SEGM:POW:CONT ON' %(self._ci))
                self._visainstrument.write('SENS%i:SEGM:ARB ON' %(self._ci))
                self._visainstrument.write('SENS%i:SEGM:X:SPAC OBAS' %(self._ci))
            return self._visainstrument.write('SENS%i:SWE:TYPE %s' %(self._ci,swtype))
        else:
            logging.error(__name__ + ' : Illegal argument %s' % swtype)
            return False
            
    def do_get_measurement_parameter(self):
        """
        Gets the measurement parameter, i.e. S11, S21, ...
        Output:
            Parameter as string "S11" ...
        """
        return self.query("CALC%i:MEAS:PAR?"%self._ci).strip()
    
    def do_set_measurement_parameter(self, mode):
        """
        Sets the measurement parameter, i.e. S11, S21, ...
        Input:
            mode: Parameter as string, e.g. "S11"
        """
        return self.write("CALC%i:MEAS:PAR %s"%(self._ci, mode))

    def do_set_trigger_source(self, source):
        """
        Set Trigger Mode

        Input:
            source (string) : EXTernal | IMMediate (internal) | MANual
        Output:
            None
        Default:
            IMMediate
        """
        logging.debug(__name__ + ' : setting trigger source to "%s"' % source)
        if source.upper() in ['EXT', 'IMM', 'MAN']:
            self._visainstrument.write('TRIG:SEQ:SOUR %s' % source.upper())
        else:
            raise ValueError('set_trigger_source(): must be EXTernal | IMMediate | MANual')

    def do_get_trigger_source(self):
        """
        Get Trigger Mode

        Input:
            None

        Output:
            source (string) : INTernal | MANual | EXTernal | BUS
        """
        logging.debug(__name__ + ' : getting trigger source')
        return str(self._visainstrument.query('TRIG:SEQ:SOUR?')).rstrip()

    def send_trigger(self):
        """
        Sends one trigger to all channels when trigger source is MANual
        """
        self._visainstrument.write('INIT:IMM')
          
    def query(self, msg):
        return self._visainstrument.query(msg)

    def write(self, msg):
        return self._visainstrument.write(msg)

    def pre_measurement(self):
        """
        Set everything needed for the measurement
        Averaging has to be enabled.
        Trigger count is set to number of averages
        """
        if not self.get_Average():
            self.set_Average(True)
            self.set_averages(1)
        self.set_count(self.do_get_averages())

    def post_measurement(self):
        """
        Bring the VNA back to a mode where it can be easily used by the operator.
        """
        self.set_sweep_mode("cont")

    def start_measurement(self):
        """
        This function is called at the beginning of each single measurement in the spectroscopy script.
        Here, it starts n sweeps, where n is the active channels trigger count.
        Also, the averages need to be reset.
        """
        self.avg_clear()
        self.set_sweep_mode("group")

    def ready(self):
        """
        This is a proxy function, returning True when the VNA is on HOLD after finishing the required number of averages .
        """
        try:  # the VNA sometimes throws an error here, we just ignore it
            return self.get_sweep_mode() == "HOLD"
        except:
            return False
    
    def reconnect(self):
        self._visainstrument = visa.instrument(self._address)

    def add_segment(self,center,span,nop,power):
        self.write("SENS:SEGM1:ADD")
        if span >= 0:
            self.write("SENS:SEGM1:FREQ:CENT %f" % center)
            self.write("SENS:SEGM1:FREQ:SPAN %f"%span)
        else:
            start, stop = center+numpy.array((-1, 1))*span/2
            self.write("SENS:SEGM1:FREQ:STAR %f" % start)
            self.write("SENS:SEGM1:FREQ:STOP %f" % stop)
        self.write("SENS:SEGM1:SWE:POIN %i"%nop)
        self.write("SENS:SEGM1:POW %f"%power)
        self.write("SENS:SEGM1 ON")
        qkit.flow.sleep(1) # The VNA needs some time to digest this...
        
        
    def delete_all_segments(self):
        self.write("SENS:SEGM:DEL:ALL")



# RS FSUP
# Andre Schneider <andre.schneider@student.kit.edu> 2014
# modified: JB <jochen.braumueller@kit.edu> 11/2016
# modified: Jan Brehm <jan.brehm@kit.edu> 02/2019
# modified: Nicolas Gosling <Nicolas.Gosling@partner.kit.edu> 04/21
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


from qkit.core.instrument_base import Instrument
from qkit import visa
import logging
from time import sleep
import numpy

class RS_FSUP(Instrument):
    '''
    This is the python driver for the Rhode&Schwarz FSUP Signal Source Analyzer.
    The command set is not complete.

    Usage:
    Initialise with
    <name> = instruments.create('<name>', address='<GPIB address>', reset=<bool>)
    
    '''

    def __init__(self, name, address, channel_index = 1):
        '''
        Initializes 

        Input:
            name (string)    : name of the instrument
            address (string) : GPIB address
        '''
        
        logging.info(__name__ + ' : Initializing instrument')
        Instrument.__init__(self, name, tags=['physical'])

        self._address = address
        self._visainstrument = visa.instrument(self._address)

        # Implement parameters

        self.add_parameter('averages', type=int,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=32767, tags=['sweep'])    

        self.add_parameter('centerfreq', type=float,
            flags=Instrument.FLAG_GETSET,
            minval=0, maxval=26.5e9,
            tags=['sweep'])

        self.add_parameter('nop', type=int,
            flags=Instrument.FLAG_GETSET,
            minval=155, maxval=30001,
            tags=['sweep'])
        self.add_parameter('freqspan', type=float,
            flags=Instrument.FLAG_GETSET,tags=['sweep'])                    

        self.add_parameter('powerunit', type=str,
            flags=Instrument.FLAG_GETSET,tags=['sweep']) 

        self.add_parameter('startfreq', type=float,
            flags=Instrument.FLAG_GETSET,tags=['sweep']) 

        self.add_parameter('stopfreq', type=float,
            flags=Instrument.FLAG_GETSET,tags=['sweep']) 

        self.add_parameter('sweeptime', type=float,
            flags=Instrument.FLAG_GETSET,tags=['sweep'])

        self.add_parameter('resolutionBW', type=float,
            flags=Instrument.FLAG_GETSET,tags=['sweep'])

        self.add_parameter('videoBW', type=float,
            flags=Instrument.FLAG_GETSET,tags=['sweep'])        
            
        self.add_parameter('sweeptime_averages', type=float,
            flags=Instrument.FLAG_GET,tags=['sweep'])

        self.add_parameter('sweeptime_auto', type=float,
            flags=Instrument.FLAG_GETSET,tags=['sweep'])    

        self.add_parameter('freqpoints', type=numpy.ndarray,
            flags=Instrument.FLAG_GET,tags=['sweep'])    

        self.add_parameter('active_traces', type=int, flags=Instrument.FLAG_GET)    
                
        # Implement functions
        self.add_function('set_continuous_sweep_mode')
        #self.add_function('set_freq_center')
        #self.add_function('set_freq_span')
        self.add_function('set_marker')
        self.add_function('start_measurement')
        self.add_function('ready')
        self.add_function('set_powerunit')
        self.add_function('get_marker_level')
        self.add_function('get_trace_name')
        self.add_function('get_y_unit')
        #self.add_function('avg_clear')
        #self.add_function('avg_status')
        
        self.get_all()
    
    def get_all(self):        
        self.get_marker_level(1)
        
        #self.get_zerospan()

    def do_set_averages(self, av):
        '''
        Set number of averages

        Input:
            av (int) : Number of averages

        Output:
            None
        '''

        self.write('AVER ON')
        self.write('AVER:AUTO OFF')
        self.write('AVER:COUN %i' % (av))
  


    def do_get_averages(self):
        '''
        Get number of averages

        Input:
            None
        Output:
            number of averages
        '''
        
        return int(self.ask('AVER:COUN?'))
    
    def do_get_freqpoints(self):
        '''
        returns an array with the frequencies of the points returned by get_trace()
        ideally suitable as x-axis for plots
        '''

        x_values=numpy.linspace(self.do_get_startfreq(), self.do_get_stopfreq(), self.do_get_nop())
        return x_values
        

    def do_set_resolutionBW(self, BW):
        '''
        sets the resolution bandwidth
        '''
        self.write('band %e'%(BW))

    def do_get_resolutionBW(self):
        '''
        gets the resolution bandwidth
        '''
        return float(self.ask('band?'))


    def get_y_unit(self, channel):
        '''
        gets the y-unit
        '''
        y_unit=self.ask('CALC:LIM1:UNIT?')[:-1]
        return str(y_unit.strip('"'))

    def do_set_videoBW(self, BW):
        '''
        sets the video bandwidth
        '''
        self.write('band:vid %e'%(BW))

    def do_get_videoBW(self):
        '''
        gets the video bandwidth
        '''
        return float(self.ask('band:vid?'))

    def do_set_sweeptime(self, sweeptime):
        '''
        sets the sweeptime
        sweeptime in seconds (e.g. 3s) or milliseconds (e.g. 50ms)
        '''
        self.write('swe:time %s'%sweeptime)
        
    def do_get_sweeptime(self):
        '''
        gets the sweeptime
        '''
        return float(self.ask('swe:time?'))

    def do_set_sweeptime_auto(self, autosweep):
        '''
        sets the sweeptime
        sweeptime in seconds (e.g. 3s) or milliseconds (e.g. 50ms)
        '''
        self.write('SWE:TIME:AUTO %s'%autosweep)
        
    def do_get_sweeptime_auto(self):
        '''
        gets the sweeptime
        '''
        return float(self.ask('SWE:TIME:AUTO?'))
    

    def do_get_sweeptime_averages(self):
        '''
        gets the sweeptime averages
        '''
        return self.do_get_sweeptime()*self.do_get_averages()

    def do_get_active_traces(self):
        '''
        gets the active channels A,B,AB for channel 1,2,1+2
        '''
        return 1

    def get_trace_name(self, trace):
        '''
        function for signal spectroscopy script to automatically generate names for trace data
        :param trace:
        :return: (string) name of trace
        '''
        return 'Spectrum'

    def do_set_centerfreq(self, centerfreq):
        '''
        sets the center frequency
        '''
        self.write('freq:cent %e'%(centerfreq))

    def do_get_centerfreq(self):
        '''
        gets the center frequency
        '''
        return float(self.ask('freq:cent?'))

    def do_set_freqspan(self, freqspan):
        '''
        sets the frequency span
        '''
        self.write('freq:span %e'%(freqspan))
    
    def do_get_freqspan(self):
        '''
        get the frequency span
        '''
        return float(self.ask('freq:span?'))

    def do_set_startfreq(self, freq):
        self.write('freq:start %e'%(freq))

    def do_get_startfreq(self):
        return float(self.ask('freq:start?'))

    def do_set_stopfreq(self, freq):
        self.write('freq:stop %e'%(freq))

    def do_get_stopfreq(self):
        return float(self.ask('freq:stop?'))
        
    def do_set_nop(self, nop):
        self.write('swe:poin %i'%(nop))

    def do_get_nop(self):
        return int(self.ask('swe:poin?'))

    def ready(self):
        '''
        This is a proxy function, returning True when the VNA has finished the required number of averages.
        '''
        status = int(self.ask('STAT:OPER:COND?')[0:-1])
        if status  == 0:
            return True
        else:
            return False
        
    def do_set_powerunit(self,unit):
        '''
        sets the unit for powers
        provide unit as a string! ("DBm")
        '''
        self.write('unit:pow %s'%(unit))
        
    def do_get_powerunit(self):
        '''
        gets the power unit for powers
        '''
        return self.ask('unit:pow?').strip()
    
    def set_marker(self,marker,frequency):
        '''
        sets marker number marker to frequency
        
        '''
        self.write('calc:mark%i:x %e'%(marker, frequency))
        self.enable_marker(marker)
        
    def get_marker(self,marker):
        '''
        gets frequency of marker
        
        '''
        return float(self.ask('calc:mark%i:x?'%(marker)))
    
    def get_marker_level(self,marker):
        '''
        gets power level of indicated marker
        
        '''
        return float(self.ask('calc:mark%i:y?'%(marker)))
        
    def set_continuous_sweep_mode(self,value):
        '''
        value='ON' Continuous sweep
        value='OFF' Single sweep
        '''
        self.write('INIT:CONT %s'%(value))
    
    def enable_marker(self,marker,state='ON'):
        '''
        ON or OFF
        '''
        self.write('CALC:MARK%i %s'%(marker,state))
    
    def sweep(self):
        '''
        perform a sweep and wait for it to finish
        '''
        self.write('INIT; *WAI')
    
    def get_trace(self, tracenumber=1):
        return self._visainstrument.query_ascii_values('trac:data? trace%i'%tracenumber)

    def start_measurement(self):
        """starts a new measurement"""
        self.write("INIT:CONT OFF")
        self.write("*CLS")
        self.write("INIT:IMM; *OPC")
        
    def get_tracedata(self,tracenumber=1):

        amp = numpy.array(self._visainstrument.query_ascii_values('trac:data? trace%i'%tracenumber))
        return amp
    
    def write(self, command):
        self._visainstrument.write(command)
    
    def ask(self,command):
        return self._visainstrument.query(command)

    def pre_measurement(self):
        pass

    def post_measurement(self):
        self.write("INIT:CONT ON")
    



# Agilent_VNA_E5071C driver, P. Macha, modified by M. Weides July 2013, J. Braumueller 2015
# Adapted to Keysight VNA by A. Schneider and L. Gurenhaupt 2016
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from qkit.core.instrument_base import Instrument
import numpy as np


class DummyVNA(Instrument):
    """
    simple dummy VNA class to test changes in the spectroscopy script without having a real device. Feel free to extend
    the functionality, for example noise that scales with power and averages.
    get_tracedata generates a random S21 trace with an overcoupled resonator dip
    frequency changes should behave like a real VNA, i.e., span is kept constant when start stop centerfreq are changed
    No Docstrings. If you don't know what the functions do, you should go into the lab first and measure with a real
    VNA.
    """
    def __init__(self, name):
        Instrument.__init__(self, name, tags=['virtual'])
        self.nop = 1001
        self.startfreq = 4e9
        self.stopfreq = 5e9
        self.sweeptime_averages = 1
        self.span = self.stopfreq - self.startfreq
        self.centerfreq = (self.startfreq + self.stopfreq) / 2
        self._ready = False
        self.add_parameter('power', type=float, minval=-85, maxval=10, units='dBm',offset=True,flags=Instrument.FLAG_GET_AFTER_SET|Instrument.FLAG_GETSET)
        self.add_function('get_freqpoints')
        self.add_function('get_tracedata')
        self.add_function('get_sweeptime_averages')
        self.add_function('pre_measurement')
        self.add_function('start_measurement')
        self.add_function('ready')
        self.add_function('post_measurement')
        self.power = 0
        
    def do_get_power(self):
        return  self.power
    
    def do_set_power(self,value):
        print("Setting VNA power to ",value)
        self.power = value

    def set_startfreq(self, startfreq):
        self.startfreq = startfreq
        self.span = self.stopfreq - self.startfreq
        self.centerfreq = (self.stopfreq + self.startfreq) / 2

    def get_startfreq(self):
        return self.startfreq

    def set_stopfreq(self, stopfreq):
        self.stopfreq = stopfreq
        self.span = self.stopfreq - self.startfreq
        self.centerfreq = (self.stopfreq + self.startfreq) / 2

    def get_stopfreq(self):
        return self.stopfreq

    def set_centerfreq(self, centerfreq):
        self.centerfreq = centerfreq
        self.startfreq = centerfreq - self.span / 2
        self.stopfreq = centerfreq + self.span / 2

    def get_centerfreq(self):
        return self.centerfreq

    def get_span(self):
        return self.span

    def set_span(self, span):
        self.span = span
        self.startfreq = self.centerfreq - span / 2
        self.stopfreq = self.centerfreq + span / 2

    def get_nop(self):
        return self.nop

    def set_nop(self, nop):
        self.nop = nop

    def get_freqpoints(self):
        return np.linspace(self.startfreq, self.stopfreq, self.nop)

    def get_sweeptime_averages(self,query=True):
        return self.sweeptime_averages

    def get_sweeptime(self,query=True):
        return 1

    def get_averages(self):
        return 1

    def get_Average(self):
        return False

    def get_tracedata(self, RealImag=None):

        def S21_notch(f, fr, Ql, Qc):
            return 1. - Ql / Qc / (1. + 2j * Ql * (f - fr) / fr)

        Qi = np.random.normal(2000, 500)
        Qc = np.random.normal(1500, 500)  # for a nice signal the resonance is overcoupled
        fr = np.random.normal((self.stopfreq + self.startfreq) / 2, self.span / 8)
        Ql = Qi * Qc / (Qi + Qc)
        S21_data = S21_notch(self.get_freqpoints(), fr, Ql, Qi) + np.random.normal(0., 0.01, self.nop)

        if not RealImag:
            amp = np.abs(S21_data)
            pha = np.angle(S21_data)
            return amp, pha
        else:
            I = S21_data.real
            Q = S21_data.imag
            return I, Q

    def get_all(self):
        pass

    def pre_measurement(self):
        pass

    def post_measurement(self):
        pass

    def start_measurement(self):
        pass

    def ready(self):
        self._ready = not self._ready
        return self._ready

    def avg_clear(self):
        pass
